if not SimpleSelfRebuff then return end
local SimpleSelfRebuff = SimpleSelfRebuff
SimpleSelfRebuff:_rev("$Revision: 59843 $")

-------------------------------------------------------------------------------
-- Externals
-------------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("SimpleSelfRebuff")

local STATE_UNSET   = SimpleSelfRebuff.STATE_UNSET
local STATE_BUFFED  = SimpleSelfRebuff.STATE_BUFFED
local STATE_FADING  = SimpleSelfRebuff.STATE_FADING
local STATE_MISSING = SimpleSelfRebuff.STATE_MISSING

local new = SimpleSelfRebuff.new
local del = SimpleSelfRebuff.del

-------------------------------------------------------------------------------
-- Locals
-------------------------------------------------------------------------------

local db

local bindingButton
local TEH_BUTTON_NAME = "SimpleSelfRebuff_MouseWheelBindingButton"

local pendingBuff
local boundBuff

local boundKeys = {}
local isBound = false

local updateEvents = {
	'SimpleSelfRebuff_MonitoringEnabled',
	'SimpleSelfRebuff_StateChanged',
	'PLAYER_CONTROL_GAINED',
	'PLAYER_REGEN_ENABLED',
	'PLAYER_AURAS_CHANGED'
}

-------------------------------------------------------------------------------
-- Module declaration
-------------------------------------------------------------------------------

local MouseWheelBinding = SimpleSelfRebuff:NewModule('MouseWheelBinding')

MouseWheelBinding.optionName = L["Mouse wheel bindings"]

MouseWheelBinding.options = {
	zoom = {
		type = 'toggle',
		name = L["Zoom when casting"],
		desc = L["Enable zooming when using the mousewheel to cast."],
		get = function() return db.zoom end,
		set = function(v) db.zoom = v end
	},
}

function MouseWheelBinding:OnModuleInitialize()
	self:RegisterDefaults('profile', {
		zoom = true,
	})
	db = self.db.profile
end

function MouseWheelBinding:OnModuleEnable()
	self:RegisterBucketEvent(updateEvents, 0.1, 'UpdateBinding')
	
	self:RegisterEvent('PLAYER_REGEN_DISABLED', 'SetBinding', false)
	self:RegisterEvent('SimpleSelfRebuff_MonitoringDisabled', 'SetBinding', false)
	self:RegisterEvent('UPDATE_BINDINGS', 'KeyBindingChanged')
	self:KeyBindingChanged()
		
	self:UpdateBinding()
end

function MouseWheelBinding:OnModuleDisable()
	self:SetBinding(false)
end

function MouseWheelBinding:KeyBindingChanged()
	boundKeys = del(boundKeys)
	boundKeys = new("MOUSEWHEELUP", "MOUSEWHEELDOWN", "SHIFT-MOUSEWHEELUP", "SHIFT-MOUSEWHEELDOWN", GetBindingKey('SIMPLE_REBUFF'))
end

function MouseWheelBinding:UpdateBinding()
	if self.core:IsMonitoringActive() then
		pendingBuff = self.core:GetPendingBuff(pendingBuff)	
		self:SetBinding(pendingBuff)		
	else
		self:SetBinding(false)
	end
end

function MouseWheelBinding:SetBinding(buff)

	if buff then
		
		if InCombatLockdown() then
			return
		end
		
		if not bindingButton then
			bindingButton = CreateFrame("Button", TEH_BUTTON_NAME, UIParent, "SecureActionButtonTemplate")
			bindingButton:RegisterForClicks("LeftButtonUp")
			bindingButton:SetPoint("CENTER",0,0)
			bindingButton:SetScript("PreClick", function(...) return self:OnButtonPreClick(...) end)
			bindingButton:SetScript("PostClick", function(...) return self:OnButtonPostClick(...) end)
			bindingButton:Hide()
			
			self:ClearButtonBindings()
		end
		
		-- Binding
		if not isBound then
			self:Debug('Binding mousewheel')
			for i,key in ipairs(boundKeys) do
				SetOverrideBindingClick(bindingButton, true, key, TEH_BUTTON_NAME, key)
			end
			isBound = true
		end
		
		boundBuff = buff
		self:Debug('Binding for '..boundBuff.name)
		
	else
		
		if InCombatLockdown() then
			self:Debug('Plan unbinding for end of combat')
			self:ScheduleLeaveCombatAction('SetBinding', false)
		else
			if bindingButton and isBound then
				self:Debug('Unbinding mousewheel')
				self:ClearButtonBindings()
				ClearOverrideBindings(bindingButton)
			end
			isBound = false
			boundBuff = nil
		end
		
	end

end

function MouseWheelBinding:OnButtonPreClick()
	if not InCombatLockdown() then
		self:ClearButtonBindings()
		if boundBuff then
			if not boundBuff:IsInCooldown() then
				self:Debug('Setting up casting for '..boundBuff.name)
				boundBuff:SetupSecureButton(bindingButton)
			else
				self:Debug(boundBuff.name.." in cooldown")
			end
		else
			self:Debug("No bound buff")
		end
	end
end

function MouseWheelBinding:OnButtonPostClick(button, key)
	if db.zoom then
		if key == 'MOUSEWHEELUP' then
			CameraZoomIn(1)
		elseif key == 'MOUSEWHEELDOWN' then
			CameraZoomOut(1)
		end
	end

	if boundBuff and not InCombatLockdown() then
		if self:ClearButtonBindings() then
			boundBuff = nil
		end	
	end
end

function MouseWheelBinding:ClearButtonBindings()
	if bindingButton and bindingButton:GetAttribute('type') ~= nil then
		self:Debug('Clearing up binding button')
		bindingButton:SetAttribute('type', nil)
		bindingButton:SetAttribute('spell', nil)			
		bindingButton:SetAttribute('unit', nil)			
		bindingButton:SetAttribute('target-slot', nil)					
		return true
	end
end

function MouseWheelBinding:DummyBinding()
	self:Debug("Do nothing")
end
